@@ -377,7 +377,7 @@ namespace dxvk {
 
   void D3D11CommonTexture::SetDebugName(const char* pName) {
     if (m_image) {
-      m_device->GetContext()->InjectCs([
+      m_device->GetContext()->InjectCs(DxvkCsQueue::HighPriority, [
         cImage  = m_image,
         cName   = std::string(pName ? pName : "")
       ] (DxvkContext* ctx) {
@@ -387,7 +387,7 @@ namespace dxvk {
 
     if (m_mapMode == D3D11_COMMON_TEXTURE_MAP_MODE_STAGING) {
       for (uint32_t i = 0; i < m_buffers.size(); i++) {
-        m_device->GetContext()->InjectCs([
+        m_device->GetContext()->InjectCs(DxvkCsQueue::HighPriority, [
           cBuffer = m_buffers[i].buffer,
           cName   = std::string(pName ? pName : "")
         ] (DxvkContext* ctx) {
@@ -827,8 +827,9 @@ namespace dxvk {
 
     // Filter out unnecessary flags. Transfer operations
     // are handled by the backend in a transparent manner.
-    Usage &= ~(VK_IMAGE_USAGE_TRANSFER_DST_BIT
-             | VK_IMAGE_USAGE_TRANSFER_SRC_BIT);
+    Usage &= VK_IMAGE_USAGE_SAMPLED_BIT | VK_IMAGE_USAGE_STORAGE_BIT
+      | VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT
+      | VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT;
 
     // Storage images require GENERAL.
     if (Usage & VK_IMAGE_USAGE_STORAGE_BIT)
@@ -837,7 +838,7 @@ namespace dxvk {
     // Also use GENERAL if the image cannot be rendered to. This
     // should not harm any hardware in practice and may avoid some
     // redundant layout transitions for regular textures.
-    if (Usage == VK_IMAGE_USAGE_SAMPLED_BIT)
+    if (!(Usage & ~VK_IMAGE_USAGE_SAMPLED_BIT))
       return VK_IMAGE_LAYOUT_GENERAL;
 
     // If the image is used only as an attachment, we never
