@@ -117,8 +117,7 @@ namespace dxvk {
     DxvkNameList layerList;
     m_extensionSet = DxvkNameSet();
 
-    bool enablePerfEvents = false;
-    bool enableValidation = false;
+    DxvkDebugFlags debugFlags = 0u;
 
     if (args.instance) {
       m_extensionNames = DxvkNameList(args.extensionCount, args.extensionNames);
@@ -133,15 +132,17 @@ namespace dxvk {
       // This extension adds additional overhead to winevulkan.
       std::string debugEnv = env::getEnvVar("DXVK_DEBUG");
 
-      enablePerfEvents = debugEnv == "markers";
-      enableValidation = debugEnv == "validation";
+      if (debugEnv == "validation")
+        debugFlags.set(DxvkDebugFlag::Validation);
+      else if (debugEnv == "markers")
+        debugFlags.set(DxvkDebugFlag::Capture, DxvkDebugFlag::Markers);
+      else if (debugEnv == "capture" || m_options.enableDebugUtils)
+        debugFlags.set(DxvkDebugFlag::Capture);
 
-      bool enableDebug = enablePerfEvents || enableValidation || m_options.enableDebugUtils;
-
-      if (enableDebug) {
+      if (!debugFlags.isClear()) {
         Logger::warn("Debug Utils are enabled. May affect performance.");
 
-        if (enableValidation) {
+        if (debugFlags.test(DxvkDebugFlag::Validation)) {
           const char* layerName = "VK_LAYER_KHRONOS_validation";
           DxvkNameSet layers = DxvkNameSet::enumInstanceLayers(m_vkl);
 
@@ -157,7 +158,7 @@ namespace dxvk {
 
       // Get set of extensions to enable based on available
       // extensions and extension providers.
-      auto extensionInfos = getExtensionList(m_extensions, enableDebug);
+      auto extensionInfos = getExtensionList(m_extensions, !debugFlags.isClear());
       DxvkNameSet extensionsAvailable = DxvkNameSet::enumInstanceExtensions(m_vkl);
 
       if (!extensionsAvailable.enableExtensions(extensionInfos.size(), extensionInfos.data(), &m_extensionSet))
@@ -183,7 +184,7 @@ namespace dxvk {
       appInfo.pApplicationName      = appName.c_str();
       appInfo.applicationVersion    = flags.raw();
       appInfo.pEngineName           = "DXVK";
-      appInfo.engineVersion         = VK_MAKE_API_VERSION(0, 2, 5, 3);
+      appInfo.engineVersion         = VK_MAKE_API_VERSION(0, 2, 6, 1);
       appInfo.apiVersion            = VK_MAKE_API_VERSION(0, 1, 3, 0);
 
       VkInstanceCreateInfo info = { VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO };
@@ -202,7 +203,7 @@ namespace dxvk {
     // Create the Vulkan instance loader
     m_vki = new vk::InstanceFn(m_vkl, !args.instance, instance);
 
-    if (enableValidation) {
+    if (debugFlags.test(DxvkDebugFlag::Validation)) {
       VkDebugUtilsMessengerCreateInfoEXT messengerInfo = { VK_STRUCTURE_TYPE_DEBUG_UTILS_MESSENGER_CREATE_INFO_EXT };
       messengerInfo.messageSeverity = VK_DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT
                                     | VK_DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT
@@ -214,6 +215,9 @@ namespace dxvk {
       if (m_vki->vkCreateDebugUtilsMessengerEXT(m_vki->instance(), &messengerInfo, nullptr, &m_messenger))
         Logger::err("DxvkInstance::createInstance: Failed to create debug messenger, proceeding without.");
     }
+
+    // Write back debug flags
+    m_debugFlags = debugFlags;
   }
 
 
