@@ -11,12 +11,14 @@
 
 #include "../util_env.h"
 
+#include "../sha1/sha1_util.h"
+
 namespace dxvk {
 
   using ProfileList = std::vector<std::pair<const char*, Config>>;
 
 
-  const static ProfileList g_profiles = {{
+  const static ProfileList g_profiles = {
     /**********************************************/
     /* D3D12 GAMES (vkd3d-proton with dxvk dxgi)  */
     /**********************************************/
@@ -48,6 +50,11 @@ namespace dxvk {
     { R"(\\BrightMemory_EP1-Win64-Shipping\.exe$)", {{
       { "dxvk.hideIntegratedGraphics",      "True"  },
     }} },
+    /* AC Shadows: Uses composition swapchain to   *
+     * check for HDR support                       */
+    { R"(\\ACShadows\.exe$)", {{
+      { "dxgi.enableDummyCompositionSwapchain", "True" }
+    }} },
 
     /**********************************************/
     /* D3D11 GAMES                                */
@@ -80,7 +87,6 @@ namespace dxvk {
     /* The Evil Within: Submits command lists     *
      * multiple times                             */
     { R"(\\EvilWithin(Demo)?\.exe$)", {{
-      { "d3d11.dcSingleUseMode",            "False" },
       { "d3d11.cachedDynamicResources",     "vi"   },
     }} },
     /* Far Cry 3: Assumes clear(0.5) on an UNORM  *
@@ -159,13 +165,10 @@ namespace dxvk {
     { R"(\\NieR Replicant ver\.1\.22474487139\.exe)", {{
       { "d3d11.cachedDynamicResources",     "vi"   },
     }} },
-    /* SteamVR performance test                   */
-    { R"(\\vr\.exe$)", {{
-      { "d3d11.dcSingleUseMode",            "False" },
-    }} },
     /* Hitman 2 - requires AGS library      */
     { R"(\\HITMAN2\.exe$)", {{
       { "dxgi.customVendorId",              "10de" },
+      { "d3d11.cachedDynamicResources",     "c"    },
     }} },
     /* Modern Warfare Remastered                  */
     { R"(\\h1(_[ms]p64_ship|-mod)\.exe$)", {{
@@ -289,12 +292,11 @@ namespace dxvk {
     /* Final Fantasy XV: VXAO does thousands of   *
      * draw calls with the same UAV bound         */
     { R"(\\ffxv_s\.exe$)", {{
-      { "d3d11.ignoreGraphicsBarriers",     "True" },
+      { "d3d11.relaxedGraphicsBarriers",    "True" },
     }} },
     /* God of War - relies on NVAPI/AMDAGS for    *
      * barrier stuff, needs nvapi for DLSS        */
     { R"(\\GoW\.exe$)", {{
-      { "d3d11.ignoreGraphicsBarriers",     "True" },
       { "d3d11.relaxedBarriers",            "True" },
       { "dxgi.hideNvidiaGpu",               "False" },
       { "dxgi.maxFrameLatency",             "1"    },
@@ -332,7 +334,7 @@ namespace dxvk {
      * presumably for culling, which doesn't play *
      * nicely with D3D11 without vendor libraries */
     { R"(\\Stray-Win64-Shipping\.exe$)", {{
-      { "d3d11.ignoreGraphicsBarriers",     "True" },
+      { "d3d11.relaxedGraphicsBarriers",    "True" },
     }} },
     /* Metal Gear Solid V: Ground Zeroes          *
      * Texture quality can break at high vram     */
@@ -349,11 +351,6 @@ namespace dxvk {
     { R"(\\MSFC\.exe$)", {{
       { "dxgi.maxFrameRate",                "60" },
     }} },
-    /* Cardfight!! Vanguard Dear Days:            *
-     * Submits command lists multiple times       */
-    { R"(\\VG2\.exe$)", {{
-      { "d3d11.dcSingleUseMode",            "False" },
-    }} },
     /* Battlefield: Bad Company 2                 *
      * Gets rid of black flickering               */
     { R"(\\BFBC2Game\.exe$)", {{
@@ -382,10 +379,6 @@ namespace dxvk {
     { R"(\\CrashBandicootNSaneTrilogy\.exe$)", {{
       { "dxgi.syncInterval",                "1"   },
     }} },
-    /* SnowRunner                                 */
-    { R"(\\SnowRunner\.exe$)", {{
-      { "d3d11.dcSingleUseMode",            "False" },
-    }} },
     /* Fallout 76
      * Game tries to be too "smart" and changes sync
      * interval based on performance (in fullscreen)
@@ -407,10 +400,6 @@ namespace dxvk {
     { R"(\\BLADESTORM Nightmare\\Launch_(EA|JP)\.exe$)", {{
       { "dxgi.maxFrameRate",                "60"  },
     }} },
-    /* Ghost Recon Wildlands                      */
-    { R"(\\GRW\.exe$)", {{
-      { "d3d11.dcSingleUseMode",            "False" },
-    }} },
     /* Vindictus d3d11 CPU bound perf, and work   *
      * around the game not properly initializing  *
      * some of its constant buffers after discard */
@@ -431,7 +420,7 @@ namespace dxvk {
      * and assumes that AMD GPUs do not expose    *
      * native command lists for AGS usage         */
     { R"(\\granblue_fantasy_relink\.exe$)", {{
-      { "d3d11.ignoreGraphicsBarriers",     "True"  },
+      { "d3d11.relaxedGraphicsBarriers",    "True"  },
       { "d3d11.exposeDriverCommandLists",   "False" },
       { "dxgi.hideNvidiaGpu",               "False" },
     }} },
@@ -475,6 +464,11 @@ namespace dxvk {
     { R"(\\FarCry(5|NewDawn)\.exe$)", {{
       { "d3d11.zeroInitWorkgroupMemory",    "True" },
     }} },
+    /* Watch Dogs 2 - ships broken compute shaders *
+     * with no barriers when they are needed       */
+    { R"(\\WatchDogs2\.exe$)", {{
+      { "d3d11.forceComputeUavBarriers",    "True" },
+    }} },
 
     /**********************************************/
     /* D3D9 GAMES                                 */
@@ -610,7 +604,7 @@ namespace dxvk {
      * Also hang when alt tabbing which seems     *
      * like a game bug that d3d9 drivers work     *
      * around.                                    */
-    { R"(\\(BF2|BF2142)\.exe$)", {{
+    { R"(\\(BF2|BF2142|PRBF2)\.exe$)", {{
       { "d3d9.deviceLossOnFocusLoss",       "True" },
       { "d3d9.countLosableResources",       "False"},
     }} },
@@ -715,7 +709,7 @@ namespace dxvk {
       { "d3d9.maxFrameRate",                "60" },
     }} },
     /* Escape from Tarkov launcher
-       Same issue as Warhammer: RoR above       */
+       Work around partial presentation issues  */
     { R"(\\BsgLauncher\.exe$)", {{
       { "d3d9.shaderModel",                 "1" },
     }} },
@@ -936,13 +930,8 @@ namespace dxvk {
     /* Delta Force: Xtreme 1 & 2                 *
      * Black screen on Alt-Tab and performance   */
     { R"(\\(DFX|dfx2)\.exe$)", {{
-      { "d3d9.deviceLossOnFocusLoss",       "True" },
       { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
-    /* The Sims 3 - Black screen on alt-tab      */
-    { R"(\\TS3(W)?\.exe$)", {{ 
-      { "d3d9.deviceLossOnFocusLoss",       "True" },
-    }} },
     /* Prototype                                 *
      * Incorrect shadows on AMD & Intel          */
     { R"(\\prototypef\.exe$)", {{ 
@@ -963,11 +952,6 @@ namespace dxvk {
     { R"(\\Dragonshard\.exe$)", {{ 
       { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
-    /* Guild Wars 1 - Alt-tab black screen when  *
-     * fullscreen with non native resolution     */
-    { R"(\\Gw\.exe$)", {{ 
-      { "d3d9.deviceLossOnFocusLoss",       "True" },
-    }} },
     /* Battle for Middle-earth 2 and expansion   *
      * Slowdowns in certain scenarios            */
     { R"(\\(The Battle for Middle-earth( \(tm\))? II( Demo)?)"
@@ -981,7 +965,6 @@ namespace dxvk {
     /* Splinter Cell Conviction - Alt-tab black  *
      * screen and unsupported GPU complaint      */
     { R"(\\conviction_game\.exe$)", {{
-      { "d3d9.deviceLossOnFocusLoss",       "True" },
       { "dxgi.customVendorId",              "10de" },
       { "dxgi.customDeviceId",              "05e0" },
       { "dxgi.customDeviceDesc",            "GeForce GTX 295" },
@@ -1036,6 +1019,40 @@ namespace dxvk {
     { R"(\\MaxPayne3\.exe$)", {{
       { "d3d9.floatEmulation",              "Strict" },
     }} },
+    /* Star Wars Empire at War & expansion         *
+     * On Intel the Water & Shader Detail option   *
+     * can't be modified. In base game the AA      *
+     * option dissapears at 2075MB vram and above  */
+    { R"(\\(StarWarsG|sweaw|swfoc)\.exe$)", {{
+      { "d3d9.customVendorId",              "1002" },
+      { "d3d9.maxAvailableMemory",          "2048" },
+      { "d3d9.memoryTrackTest",             "True" },
+    }} },
+    /* CivCity: Rome                              *
+     * Enables soft real-time shadows             */
+    { R"(\\CivCity Rome\.exe$)", {{
+      { "d3d9.customVendorId",              "10de" },
+    }} },
+    /* Silent Hill 2 (2001)                       *
+     * The Enhancements mod configures the        *
+     * swapchain to only have a single backbuffer *
+     * and then calls GetFrontBufferData after    *
+     * rendering to the backbuffer. This causes   *
+     * it to get the wrong data from              *
+     * GetFrontBufferData().                      */
+    { R"(\\sh2pc\.exe$)", {{
+      { "d3d9.extraFrontbuffer",            "True" },
+    }} },
+    /* Lego Indiana Jones: The Original Adventures *
+     * Fix UI performance                          */
+    { R"(\\LEGOIndy\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
+    }} },
+    /* Lego Batman: The Videogame                 *
+     * Fix UI performance                         */
+    { R"((\\LEGOBatman|LegoBatman\\Game)\.exe$)", {{
+      { "d3d9.cachedDynamicBuffers",        "True" },
+    }} },
 
     /**********************************************/
     /* D3D8 GAMES                                 */
@@ -1151,10 +1168,6 @@ namespace dxvk {
       { "d3d9.maxFrameRate",                  "60" },
       { "d3d8.placeP8InScratch",            "True" },
     }} },
-    /* Rise of Nations + Expansion - alt-tab crash*/
-    { R"(\\(nations|patriots)\.exe$)", {{
-      { "d3d9.deviceLossOnFocusLoss",       "True" },
-    }} },
     /* Inquisitor (2009)                          *
      * Leaks a resource when alt-tabbing          */
     { R"(\\Inquisitor\.exe$)", {{
@@ -1180,6 +1193,7 @@ namespace dxvk {
      * Broken inputs and physics above 60 FPS     */
     { R"(\\SplinterCell2\.exe$)", {{
       { "d3d9.maxFrameRate",                  "60" },
+      { "d3d8.scaleDref",                     "24" },
     }} },
     /* Chrome: Gold Edition                       *
      * Broken character model motion at high FPS  */
@@ -1193,23 +1207,72 @@ namespace dxvk {
       { "d3d9.maxFrameRate",                  "60" },
       { "d3d8.forceLegacyDiscard",          "True" },
     }} },
-  }};
+    /* Tom Clancy's Splinter Cell                 *
+     * Fixes shadow buffers and alt-tab           */
+    { R"(\\splintercell\.exe$)", {{
+      { "d3d8.scaleDref",                     "24" },
+      { "d3d8.shadowPerspectiveDivide",     "True" },
+      { "d3d9.deviceLossOnFocusLoss",       "True" },
+    }} },
+    /* Trainz v1.3 (2001)                         *
+     * Fixes black screen after alt-tab           */
+    { R"(\\bin\\trainz\.exe$)", {{
+      { "d3d9.deviceLossOnFocusLoss",       "True" },
+    }} },
+  };
 
 
-  const static ProfileList g_deckProfiles = {{
+  const static ProfileList g_deckProfiles = {
     /* Fallout 4: Defaults to 45 FPS on OLED, but also breaks above 60 FPS */
     { R"(\\Fallout4\.exe$)", {{
       { "dxgi.syncInterval",                "1" },
       { "dxgi.maxFrameRate",                "60" },
     }} },
-  }};
+  };
+
+
+  const static ProfileList g_hashedProfiles = {
+    /* Nothing to see here */
+  };
 
 
   const Config* findProfile(const ProfileList& profiles, const std::string& appName) {
     auto appConfig = std::find_if(profiles.begin(), profiles.end(),
       [&appName] (const std::pair<const char*, Config>& pair) {
-        std::regex expr(pair.first, std::regex::extended | std::regex::icase);
-        return std::regex_search(appName, expr);
+        // With certain locales, regex parsing will simply crash. Using regex::imbue
+        // does not resolve this; only the global locale seems to matter here. Catch
+        // bad_alloc errors to work around this for now.
+        try {
+          std::regex expr(pair.first, std::regex::extended | std::regex::icase);
+          return std::regex_search(appName, expr);
+        } catch (const std::bad_alloc& e) {
+          Logger::err(str::format("Failed to parse regular expression: ", pair.first));
+          return false;
+        }
+      });
+
+    return appConfig != profiles.end()
+      ? &appConfig->second
+      : nullptr;
+  }
+
+
+  const Config* findHashedProfile(const ProfileList& profiles, const std::string& appName) {
+    // Don't bother hashing exe names if we don't have
+    // any top-secret app profiles to begin with
+    if (profiles.empty())
+      return nullptr;
+
+    auto n = appName.find_last_of('\\') + 1u;
+
+    if (n >= appName.size())
+      return nullptr;
+
+    auto hash = Sha1Hash::compute(&appName[n], appName.size() - n).toString();
+
+    auto appConfig = std::find_if(profiles.begin(), profiles.end(),
+      [&hash] (const std::pair<const char*, Config>& pair) {
+        return hash == pair.first;
       });
 
     return appConfig != profiles.end()
@@ -1475,6 +1538,9 @@ namespace dxvk {
     if (!config)
       config = findProfile(g_profiles, appName);
 
+    if (!config)
+      config = findHashedProfile(g_hashedProfiles, appName);
+
     if (config) {
       // Inform the user that we loaded a default config
       Logger::info(str::format("Found built-in config:"));
